# coding=utf-8
import random

import mod.server.extraServerApi as serverApi
from mod.server.system.serverSystem import ServerSystem

from soldierLotteryScripts.consts import ServiceServerType, GetPlayerRandomNumber, GetPlayerNumbersCount, \
    GetPlayerNumbers, ResetPlayerNumbers, ResetAllNumbers, DoLotteryDraw, PlayerJoin, PlayerLeave


class LotteryServerSystem(ServerSystem):
    def __init__(self, namespace, systemName):
        ServerSystem.__init__(self, namespace, systemName)
        self.alertSystem = serverApi.GetSystem("neteaseAlert", "neteaseAlertDev")
        self.ListenForEvent(serverApi.GetEngineNamespace(), serverApi.GetEngineSystemName(), "ServerChatEvent", self,
                            self.OnServerChat)
        self.ListenForEvent(serverApi.GetEngineNamespace(), serverApi.GetEngineSystemName(), "AddServerPlayerEvent",
                            self, self.OnJoin)
        self.ListenForEvent(serverApi.GetEngineNamespace(), serverApi.GetEngineSystemName(), "DelServerPlayerEvent",
                            self, self.OnLeave)
        self.ListenForEvent(serverApi.GetEngineNamespace(), serverApi.GetEngineSystemName(), "DestroyBlockEvent",
                            self, self.OnDestroy)

        self.idUidMap = {}

    def OnDestroy(self, args):
        blockName = args["fullName"]
        if blockName != "minecraft:dirt":
            # 不是泥土方块则不处理
            return
        if random.random() > 0.1:
            # 如果随机数大于0.1则不处理
            return
        playerId = args["playerId"]
        uid = self.idUidMap[playerId]

        def callback(number):
            if number != -1:
                self.alertSystem.Alert(playerId, '§f本次领取的号码是 §a{}'.format(number), 3)

        self.GetPlayerRandomNumber(uid, callback)

    def OnJoin(self, args):
        playerId = args["id"]
        uid = args["uid"]
        self.idUidMap[playerId] = uid
        if not args["isTransfer"]:
            print "玩家{}加入，进行缓存".format(uid)
            self.CachePlayer(uid)

    def OnLeave(self, args):
        playerId = args["id"]
        uid = args["uid"]
        del self.idUidMap[playerId]
        if not args["isTransfer"]:
            print "玩家{}退出，删除缓存".format(uid)
            self.SavePlayer(uid)

    def OnServerChat(self, args):
        playerId = args["playerId"]
        message = args["message"]
        uid = self.idUidMap[playerId]
        if message == "cp2":
            args["cancel"] = True

            def callback(numbers):
                if numbers is None:
                    self.alertSystem.Alert(playerId, '§c获取失败，请稍后再试', 3)
                    return
                size = len(numbers)
                if size == 0:
                    self.alertSystem.Alert(playerId, '§c你还没有领取任何号码', 3)
                else:
                    self.alertSystem.Alert(playerId, '§c今日已领取{}个号码: {}'.format(size, self.FormatNumbers(numbers)), 3)

            self.GetPlayerNumbers(uid, callback)

    def FormatNumbers(self, numbers):
        text = ""
        for number in numbers:
            text += str(number) + "、"
        return text[:-1]

    def CachePlayer(self, uid):
        def callback(suc, args):
            print "缓存玩家{}: suc:{} ,args:{}".format(uid,suc, args)

        self.RequestToServiceMod(ServiceServerType, PlayerJoin, {"player": uid}, callback)

    def SavePlayer(self, uid):
        def callback(suc, args):
            print "保存玩家{}: suc:{} ,args:{}".format(uid, suc, args)

        self.RequestToServiceMod(ServiceServerType, PlayerLeave, {"player": uid}, callback)

    def GetPlayerRandomNumber(self, uid, callback):
        """
        为玩家获取一个随机彩票号码
        :param uid: 玩家UID
        :param callback:int 参数唯一，随机彩票号码，请求失败值为-1
        :return:
        """
        self.RequestToServiceMod(ServiceServerType, GetPlayerRandomNumber, {"player": uid},
                                 lambda suc, args: callback(args["msg"]) if suc else callback(-1))

    def GetPlayerNumbersCount(self, uid, callback):
        """
        获玩家已获取的号码数量
        :param uid: 玩家UID
        :param callback:int 参数唯一，玩家号码数量，请求失败值为-1
        :return:
        """
        self.RequestToServiceMod(ServiceServerType, GetPlayerNumbersCount, {"player": uid},
                                 lambda suc, args: callback(args["msg"]) if suc else callback(-1))

    def GetPlayerNumbers(self, uid, callback):
        """
        获玩家已获取的号码
        :param uid: 玩家UID
        :param callback:int 参数唯一，玩家号码数量list，请求失败值为None
        :return:
        """
        self.RequestToServiceMod(ServiceServerType, GetPlayerNumbers, {"player": uid},
                                 lambda suc, args: callback(args["msg"]) if suc else callback(None))

    def ResetPlayerNumbers(self, uid, callback):
        """
        重置玩家的所有号码
        :param uid: 玩家UID
        :param callback:int 参数唯一，int 是否成功，成功为0，错误为1，请求失败值为-1
        :return:
        """
        self.RequestToServiceMod(ServiceServerType, ResetPlayerNumbers, {"player": uid},
                                 lambda suc, args: callback(args["code"]) if suc else callback(-1))

    def ResetAllNumbers(self, callback):
        """
        重置所有玩家的号码
        :param callback:int 参数唯一，int 是否成功，成功为0，错误为1，请求失败值为-1
        :return:
        """
        self.RequestToServiceMod(ServiceServerType, ResetAllNumbers, {},
                                 lambda suc, args: callback(args["code"]) if suc else callback(-1))

    def DoLotteryDraw(self, callback):
        """
        执行发奖
        :param callback:int 参数唯一，int 是否成功，成功为0，错误为1，请求失败值为-1
        :return:
        """
        self.RequestToServiceMod(ServiceServerType, DoLotteryDraw, {},
                                 lambda suc, args: callback(args["code"]) if suc else callback(-1))
