# coding=utf-8
import time

import apolloCommon.commonNetgameApi as commonNetgameApi
import mod.server.extraServiceApi as serviceApi
from mod.server.system.serviceSystem import ServiceSystem
from soldierLotteryScripts.mysqlManager import MysqlManager

from soldierLotteryScripts import consts


def unicode_convert(inputData):
    if isinstance(inputData, dict):
        return {unicode_convert(key): unicode_convert(value) for key, value in inputData.iteritems()}
    elif isinstance(inputData, list):
        return [unicode_convert(element) for element in inputData]
    elif isinstance(inputData, tuple):
        tmp = [unicode_convert(element) for element in inputData]
        return tuple(tmp)
    elif isinstance(inputData, unicode):
        return inputData.encode('utf-8')
    else:
        return inputData


class LotteryServiceSystem(ServiceSystem):

    def __init__(self, namespace, systemName):
        ServiceSystem.__init__(self, namespace, systemName)
        self.mMysqlManager = MysqlManager(self)
        self.timer = commonNetgameApi.AddRepeatedTimer(1.0, self.CheckTime)
        self.RegisterRpcMethodForMod(consts.GetPlayerRandomNumber, self.OnGetPlayerRandomNumber)
        self.RegisterRpcMethodForMod(consts.GetPlayerNumbersCount, self.OnGetPlayerNumbersCount)
        self.RegisterRpcMethodForMod(consts.GetPlayerNumbers, self.OnGetPlayerNumbers)
        self.RegisterRpcMethodForMod(consts.ResetPlayerNumbers, self.OnResetPlayerNumbers)
        self.RegisterRpcMethodForMod(consts.ResetAllNumbers, self.OnResetAllNumbers)
        self.RegisterRpcMethodForMod(consts.DoLotteryDraw, self.OnDoLotteryDraw)
        self.RegisterRpcMethodForMod(consts.PlayerJoin, self.OnJoin)
        self.RegisterRpcMethodForMod(consts.PlayerLeave, self.OnLeave)

    def GetPlayerRandomNumber(self, uid):
        lottery = self.mMysqlManager.GetUnfinishedLottery()
        if lottery == -1:
            return -1
        return self.mMysqlManager.GetPlayerRandomNumber(uid, lottery)

    def GetPlayerNumbersAmount(self, uid):
        lottery = self.mMysqlManager.GetUnfinishedLottery()
        if lottery == -1:
            return -1
        return self.mMysqlManager.GetPlayerNumbersAmount(uid, lottery)

    def GetPlayerNumbers(self, uid):
        lottery = self.mMysqlManager.GetUnfinishedLottery()
        if lottery == -1:
            return None
        return self.mMysqlManager.GetPlayerNumbers(uid, lottery)

    def ResetAllNumbers(self):
        lottery = self.mMysqlManager.GetUnfinishedLottery()
        if lottery == -1:
            return False
        return self.mMysqlManager.ResetAllNumbers(lottery)

    def ResetPlayerNumbers(self, uid):
        lottery = self.mMysqlManager.GetUnfinishedLottery()
        if lottery == -1:
            return False
        return self.mMysqlManager.ResetPlayerNumbers(uid, lottery)

    def OnGetPlayerRandomNumber(self, serverId, callbackId, args):
        player = args["player"]
        lottery = self.mMysqlManager.GetUnfinishedLottery()
        if lottery == -1:
            eventData = {
                "code": 1,
                "msg": -1
            }
        else:
            eventData = {
                "code": 0,
                "msg": self.mMysqlManager.GetPlayerRandomNumber(player, lottery)
            }

        self.ResponseToServer(serverId, callbackId, eventData)

    def OnGetPlayerNumbersCount(self, serverId, callbackId, args):
        player = args["player"]
        lottery = self.mMysqlManager.GetUnfinishedLottery()
        if lottery == -1:
            eventData = {
                "code": 1,
                "msg": -1
            }
        else:
            eventData = {
                "code": 0,
                "msg": self.mMysqlManager.GetPlayerNumbersAmount(player, lottery)
            }

        self.ResponseToServer(serverId, callbackId, eventData)

    def OnGetPlayerNumbers(self, serverId, callbackId, args):
        player = args["player"]
        lottery = self.mMysqlManager.GetUnfinishedLottery()
        if lottery == -1:
            eventData = {
                "code": 1,
                "msg": -1
            }
        else:
            eventData = {
                "code": 1,
                "msg": self.mMysqlManager.GetPlayerNumbers(player, lottery)
            }

        self.ResponseToServer(serverId, callbackId, eventData)

    def OnResetPlayerNumbers(self, serverId, callbackId, args):
        player = args["player"]
        lottery = self.mMysqlManager.GetUnfinishedLottery()
        code = 0 if self.mMysqlManager.ResetPlayerNumbers(player, lottery) else 1
        self.ResponseToServer(serverId, callbackId, {"code": code})

    def OnResetAllNumbers(self, serverId, callbackId, args):
        lottery = self.mMysqlManager.GetUnfinishedLottery()
        self.mMysqlManager.ResetAllNumbers(lottery)
        code = 0
        self.ResponseToServer(serverId, callbackId, {"code": code})

    def OnDoLotteryDraw(self, serverId, callbackId, args):
        lottery = self.mMysqlManager.GetUnfinishedLottery()
        self.mMysqlManager.FinishLottery(lottery)
        code = 0
        self.ResponseToServer(serverId, callbackId, {"code": code})

    def OnJoin(self, serverId, callbackId, args):
        player = args["player"]
        self.mMysqlManager.GetPlayerCache(player)
        code = 0
        self.ResponseToServer(serverId, callbackId, {"code": code})

    def OnLeave(self, serverId, callbackId, args):
        player = args["player"]
        self.mMysqlManager.RemovePlayerCache(player)
        code = 0
        self.ResponseToServer(serverId, callbackId, {"code": code})

    def Destroy(self):
        self.mMysqlManager.Destroy()
        commonNetgameApi.CancelTimer(self.timer)

    def CheckTime(self):
        currentTime = time.strftime("%H:%M:%S", time.localtime())
        if currentTime != "20:00:00":
            return
        self.FinishLottery()

    def FinishLottery(self):
        lottery = self.mMysqlManager.GetUnfinishedLottery()
        if lottery == -1:
            print "开奖失败，没有获取到未开奖的彩票"
        self.mMysqlManager.FinishLottery(lottery)

    def SendRewards(self, players, numbers, super=None):
        mailSystem = serviceApi.GetSystem("neteaseAnnounce", "neteaseAnnounceService")
        if super:
            mailSystem.OutSendMailToMany([super], "恭喜中超级大奖", "恭喜你获得本期特别大奖！邮件附件内容：绿宝石*100", [{
                "itemName": "minecraft:emerald",
                "count": 100,
                "auxValue": 0
            }], 86400, "彩票系统")
        content = "本期特别大奖由{}获得。另外本期中奖号码：{}。恭喜你，在本期抽奖活动中中奖，请再接再厉。邮件附件内容：黑曜石*5".format(super, self.FormatNumbers(
            numbers)) if super else "本期中奖号码：{}。恭喜你，在本期抽奖活动中中奖，请再接再厉。邮件附件内容：黑曜石*5".format(self.FormatNumbers(numbers))
        itemDict = {
            "itemName": "minecraft:obsidian",
            "count": 5,
            "auxValue": 0
        }
        self.BroadcastToService(consts.AfterLotteryDrawEvent, {"players": players, "numbers": numbers})
        print "向玩家{}发送中奖邮件".format(players)
        if not players:
            return
        mailSystem.OutSendMailToMany(players, "恭喜中奖", content, [itemDict], 86400, "彩票系统")

    def SendNonRewards(self, players, numbers, super=None):
        mailSystem = serviceApi.GetSystem("neteaseAnnounce", "neteaseAnnounceService")
        content = "本期特别大奖由{}获得。另外本期中奖号码：{}。很遗憾，在本期抽奖活动中未能中奖，请继续努力。邮件附件内容：铁锭*3".format(super, self.FormatNumbers(
            numbers)) if super else "本期中奖号码：{}。很遗憾，在本期抽奖活动中未能中奖，请继续努力。邮件附件内容：铁锭*3".format(
            self.FormatNumbers(numbers))
        itemDict = {
            "itemName": "minecraft:iron_ingot",
            "count": 3,
            "auxValue": 0
        }
        print "向玩家{}发送未中奖邮件".format(players)
        if not players:
            return
        mailSystem.OutSendMailToMany(players, "与奖品擦肩而过", content, [itemDict], 86400, "彩票系统")

    def FormatNumbers(self, numbers):
        text = ""
        for number in numbers:
            text += str(number) + ","
        return text[:-1]
